#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "rssma.h"
#include "RSSMA_AM_Example.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rssma_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rssma_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rssma_close(io); \
            io = NULL;\
            return -1; \
        } \
    }
    
/* Globals */
static int lfg1Panel, panelHandle, lfg2panel;
static int noisepanel;

ViBoolean grfState;
ViReal64  grfLevel, grfFrequency, gbandwidth,
          glfGen1Freq, glfGen1Depth, glfGen2Freq, grise, 
          gfall, ghigh, glow, gamDepth;
ViInt32   gamSource, gnoiseDistribution, gshape, gnoiseBwidLim, gamState;
ViStatus  error;
ViSession io = NULL;
ViChar    error_message[256];
ViChar    error_buffer[256];

/* Local Functions */
int initialize (void);
void ShapeChange (void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((lfg1Panel = LoadPanel (0, "RSSMA_AM_Example.uir", LFG1_PANEL)) < 0)
        return -1;
    if ((panelHandle = LoadPanel (0, "RSSMA_AM_Example.uir", PANEL)) < 0)
        return -1;
    if ((lfg2panel = LoadPanel (0, "RSSMA_AM_Example.uir", LFG2PANEL)) < 0)
        return -1;
    if ((noisepanel = LoadPanel (0, "RSSMA_AM_Example.uir", NOISEPANEL)) < 0)
        return -1;
    
    /* Default Values */
    grfState           = VI_FALSE;
    grfLevel           = -30.0;
    grfFrequency       = 1E9;
    gbandwidth         = 100e3;
    glfGen1Freq        = 1e3;
    glfGen1Depth       = 30.0;
    glfGen2Freq        = 1e3;
    grise              = 1e-6; 
    gfall              = 1e-6;
    ghigh              = 1e-6;
    glow               = 1e-6;
    gamSource          = 0;
    gnoiseDistribution = 0;
    gshape             = 0;
    gnoiseBwidLim      = 0;
    gamState           = 0;
	gamDepth		   = 30.0;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    DiscardPanel (noisepanel);
    DiscardPanel (lfg2panel);
    DiscardPanel (lfg1Panel);
    
    return 0;
}

/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            /* Initialize driver */
            if (io == VI_NULL)
            {
                if (initialize() < VI_SUCCESS) break;
            }

            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
            ProcessDrawEvents ();

            SetWaitCursor(1);

            /* Get values from controls */
            GetCtrlVal (panelHandle, PANEL_RF_FREQ, &grfFrequency);
            GetCtrlVal (panelHandle, PANEL_RF_LEVEL, &grfLevel);
            GetCtrlVal (panelHandle, PANEL_RF_STATE, &grfState);
            GetCtrlVal (panelHandle, PANEL_AM_STATE, &gamState);
            GetCtrlVal (panelHandle, PANEL_AM_DEPTH, &gamDepth);			
            GetCtrlIndex (panelHandle, PANEL_AM_SOURCE, &gamSource);
            
            /* Configure RF output part */
            CHECKERR (rssma_confRFFreq(io, grfFrequency, 0.0, VI_FALSE, 0.0));
            CHECKERR (rssma_confRFLevel (io, grfLevel, 0.0, 30.0, VI_FALSE, 0.0,
                                        RSSMA_OUTPUT_AMODE_AUTO,
                                        RSSMA_POWER_LEVEL_RCL_EXCLUDE));

            /* According to selected source configure source parameters. */
            if (gamSource)
            {
	            /* Configure common AM settings */
				CHECKERR (rssma_configAM (io, gamState, RSSMA_AM_SOURCE_INT, gamDepth, 
										  RSSMA_AM_EXTERNAL_COUPLING_AC, gamSource - 1,
										  glfGen1Depth, glfGen1Freq, gshape, glfGen2Freq)); 									  
			
               if ((gamSource == 1) || (gamSource == 3) || (gamSource == 5)) // LF Gen 1
               {
                   CHECKERR (rssma_confLFFreq (io, VI_TRUE, 1, glfGen1Freq, 1.0));
               }

               if ((gamSource == 2) || (gamSource == 3) || (gamSource == 6)) // LF Gen 2
               {
                   CHECKERR (rssma_confLFFreq (io, VI_TRUE, 2, glfGen2Freq, 1.0));
                   CHECKERR (rssma_SetLFGenShape (io, gshape));
                   if (gshape == 3) // trapeze
                   {
                       CHECKERR (rssma_SetLFGenShapeTrapeze (io, RSSMA_LFOUTPUT_SHAPE_TRAPEZE_FALL,
                                                             gfall));
                       CHECKERR (rssma_SetLFGenShapeTrapeze (io, RSSMA_LFOUTPUT_SHAPE_TRAPEZE_RISE,
                                                             grise));
                       CHECKERR (rssma_SetLFGenShapeTrapeze (io, RSSMA_LFOUTPUT_SHAPE_TRAPEZE_HIGH,
                                                             ghigh));
                       CHECKERR (rssma_SetLFGenShapeTrapeze (io, RSSMA_LFOUTPUT_SHAPE_TRAPEZE_LOW,
                                                             glow));
       
                   }
               }
               if ((gamSource == 4) || (gamSource == 5) || (gamSource == 6)) // Noise
               {
                   CHECKERR (rssma_confNoise (io, gnoiseBwidLim, gbandwidth, gnoiseDistribution));
               }
            }
			else
			{
				/* If source None is selected, sets this parameters. */				
				CHECKERR (rssma_SetAMSource(io, RSSMA_AM_SOURCE_INT));
				CHECKERR (rssma_SetAMDepth(io, gamDepth));
				CHECKERR (rssma_SetAMState (io, gamState));
			}

            /* Turn On/Off RF output at the end of sequence */
            CHECKERR (rssma_SetOutputState(io, grfState));

            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);

            SetWaitCursor(0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: OK - LF Gen 1 Settings                                          */
/* Purpose:  Stores LF Gen 1 values into variables                           */
/*===========================================================================*/
int CVICALLBACK Lfgen1Ok (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            GetCtrlVal (lfg1Panel, LFG1_PANEL_LFGEN1_FREQ, &glfGen1Freq); 
            GetCtrlVal (lfg1Panel, LFG1_PANEL_LFGEN1_DEPTH, &glfGen1Depth); 
            HidePanel (lfg1Panel);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: OK - LF Gen 2 Settings                                          */
/* Purpose:  Stores LF Gen 2 values into variables                           */
/*===========================================================================*/
int CVICALLBACK Lfgen2Ok (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            GetCtrlVal (lfg2panel, LFG2PANEL_LFGEN2_FREQ, &glfGen2Freq); 
            GetCtrlIndex (lfg2panel, LFG2PANEL_LFGEN2_SHAPE, &gshape);
            if (gshape == 3)
            {
                GetCtrlVal (lfg2panel, LFG2PANEL_LFGEN2_RISE, &grise);
                GetCtrlVal (lfg2panel, LFG2PANEL_LFGEN2_FALL, &gfall);
                GetCtrlVal (lfg2panel, LFG2PANEL_LFGEN2_HIGH, &ghigh);
                GetCtrlVal (lfg2panel, LFG2PANEL_LFGEN2_LOW, &glow);
            }

            HidePanel (lfg2panel);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: OK - Noise Settings                                             */
/* Purpose:  Stores Noise values into variables                              */
/*===========================================================================*/
int CVICALLBACK NoiseOk (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            GetCtrlVal (noisepanel, NOISEPANEL_NOISE_DISTRIBUTION, &gnoiseDistribution); 
            GetCtrlVal (noisepanel, NOISEPANEL_NOISE_BWID_LIM, &gnoiseBwidLim);
            
            if (gnoiseBwidLim)
                GetCtrlVal (noisepanel, NOISEPANEL_NOISE_BWID, &gbandwidth);
            HidePanel (noisepanel);

            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Source Changed                                                  */
/* Purpose:  This is a callback function of chaging FM source.               */
/*===========================================================================*/
int CVICALLBACK SourceChanged (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            
            GetCtrlIndex (panel, control, &gamSource);

            switch (gamSource)
            {
                case 0:
                    SetCtrlAttribute (panel, PANEL_LFGEN1_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panel, PANEL_LFGEN2_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panel, PANEL_NOISE_BUTTON, ATTR_DIMMED, 1);
                    break;
                case 1: // LF Gen 1
                    SetCtrlAttribute (panel, PANEL_LFGEN1_BUTTON, ATTR_DIMMED, 0);
                    SetCtrlAttribute (panel, PANEL_LFGEN2_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panel, PANEL_NOISE_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (lfg1Panel, LFG1_PANEL_LFGEN1_DEPTH, ATTR_DIMMED, 1);
                    break;
                case 2: // LF Gen 2
                    SetCtrlAttribute (panel, PANEL_LFGEN1_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panel, PANEL_LFGEN2_BUTTON, ATTR_DIMMED, 0);
                    SetCtrlAttribute (panel, PANEL_NOISE_BUTTON, ATTR_DIMMED, 1);
                    ShapeChange ();
                    break;
                case 3: // LF Gen 1 + 2
                    SetCtrlAttribute (panel, PANEL_LFGEN1_BUTTON, ATTR_DIMMED, 0);
                    SetCtrlAttribute (panel, PANEL_LFGEN2_BUTTON, ATTR_DIMMED, 0);
                    SetCtrlAttribute (panel, PANEL_NOISE_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (lfg1Panel, LFG1_PANEL_LFGEN1_DEPTH, ATTR_DIMMED, 0);
                    ShapeChange ();
                    break;
                case 4:
                    SetCtrlAttribute (panel, PANEL_LFGEN1_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panel, PANEL_LFGEN2_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panel, PANEL_NOISE_BUTTON, ATTR_DIMMED, 0);
                    break;
                case 5:
                    SetCtrlAttribute (panel, PANEL_LFGEN1_BUTTON, ATTR_DIMMED, 0);
                    SetCtrlAttribute (panel, PANEL_LFGEN2_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panel, PANEL_NOISE_BUTTON, ATTR_DIMMED, 0);
                    SetCtrlAttribute (lfg1Panel, LFG1_PANEL_LFGEN1_DEPTH, ATTR_DIMMED, 1);
                    break;
                case 6:
                    SetCtrlAttribute (panel, PANEL_LFGEN1_BUTTON, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panel, PANEL_LFGEN2_BUTTON, ATTR_DIMMED, 0);
                    SetCtrlAttribute (panel, PANEL_NOISE_BUTTON, ATTR_DIMMED, 0);
                    break;
                    
            }
            
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Shape Changed                                                   */
/* Purpose:  This is a callback function of chaging LF Gen 2 shape           */
/*===========================================================================*/
int CVICALLBACK ShapeChanged (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
                ShapeChange ();
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Noise Limitation                                                */
/* Purpose:  This is a callback function of chaging Noise limitation         */
/*===========================================================================*/
int CVICALLBACK NoiseLimitationState (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            GetCtrlVal (noisepanel, NOISEPANEL_NOISE_BWID_LIM, &gnoiseBwidLim);
            SetCtrlAttribute (noisepanel, NOISEPANEL_NOISE_BWID, ATTR_DIMMED, !gnoiseBwidLim);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Show LF Gen 2                                                   */
/* Purpose:  This is a callback function of LF Gen 2 Settings button.        */
/*===========================================================================*/
int CVICALLBACK Lfgen2Button (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            DisplayPanel (lfg2panel);

            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Show Noise                                                      */
/* Purpose:  This is a callback function of Noise Settings button. Shows     */
/*           noise settings panel.                                           */
/*===========================================================================*/
int CVICALLBACK NoiseButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            DisplayPanel (noisepanel);
            
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Show LF Gen 1                                                   */
/* Purpose:  This is a callback function of LF Gen 1 Settings button.        */
/*===========================================================================*/
int CVICALLBACK Lfgen1Button (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            DisplayPanel (lfg1Panel);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SMA Signal Generator\n"
                              "instrument driver provided in the package.\n");
}

/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            if (io) rssma_close (io);
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;


    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssma_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssma_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);

    SetWaitCursor(0);
    return 0;
}

/*===========================================================================*/
/* Function: Shape Change                                                    */
/* Purpose:  This function shows/hides controls on LF Gen 2 panel according  */
/*           to selected shape                                               */
/*===========================================================================*/
void ShapeChange (void)
{
    int dimmed;
    
    GetCtrlIndex (lfg2panel, LFG2PANEL_LFGEN2_SHAPE, &gshape);
    
    dimmed = (gshape != 3);
    
    SetCtrlAttribute (lfg2panel, LFG2PANEL_LFGEN2_RISE, ATTR_DIMMED, dimmed);
    SetCtrlAttribute (lfg2panel, LFG2PANEL_LFGEN2_FALL, ATTR_DIMMED, dimmed);
    SetCtrlAttribute (lfg2panel, LFG2PANEL_LFGEN2_HIGH, ATTR_DIMMED, dimmed);
    SetCtrlAttribute (lfg2panel, LFG2PANEL_LFGEN2_LOW, ATTR_DIMMED, dimmed);
}

